import os
import sys
import json
from ec2ools.aws import (
    client, metadata)
import jmespath

from pprint import pprint
IS_PY2=False

if sys.version_info[0] < 3:
    IS_PY2=True


def sort_instances_by_launch(instances, reverse=False):
    """
    """
    instances.sort(key=lambda x: (x['LaunchTime'], x['InstanceId']), reverse=reverse )
    return instances


class AsgBase:

    def get_asg_by_instance(self, instance_id=None):
        """
        """
        if not instance_id:
            instance_id = metadata('instance-id')

        res = client('autoscaling').describe_auto_scaling_instances(
            InstanceIds=[instance_id])

        print(res)
        if len(res.get('AutoScalingInstances')) <= 0:
            return

        asg_info = res.get('AutoScalingInstances')[0]

        return asg_info


class Asg(AsgBase):

    def __init__(self, asg_name):
        self.asg_name = asg_name

    def get_instances(self):
        """
        """

        res = client('autoscaling').describe_auto_scaling_groups(
            AutoScalingGroupNames=[self.asg_name])

        if len(res.get('AutoScalingGroups', [])) <= 0:
            return

        inst = res.get('AutoScalingGroups')[0].get('Instances')

        ids = [
            i['InstanceId']
            for i in inst
        ]

        res = client('ec2').describe_instances(
            InstanceIds=ids)


        try:
            instances = jmespath.search("Reservations[].Instances[]", res)
            return instances
        except Exception as e:
            print(str(e))


    def is_oldest_instance(self, instance_id=None):
        """
        """
        if not instance_id:
            instance_id = metadata('instance-id')

        info = self.get_asg_by_instance(instance_id)

        asg_name = info['AutoScalingGroupName']

        inst = self.get_instances(asg_name)

        inst = sort_instances_by_launch(inst)

        if inst[0]['InstanceId'] == instance_id:
            return True
        return False





import sys
import os
from ec2ools import aws
from ec2ools.aws import MetaService


class EipBase:

    def allocate_eip(self, alloc_id):
        """
        """
        meta = MetaService()
        instance_id = meta.query_meta('instance-id')
        eip_info = self._query_eip(alloc_id)

        if not eip_info:
            raise Exception("Invalid AllocationID")

        aws.client('ec2').associate_address(
            InstanceId=instance_id,
            AllocationId=eip_info['AllocationId'])

    def _query_eip(self, alloc_id):
        """ Query EIP record by allocation ID

        Args:
            alloc_id (str): Allocation ID of path to file containing Allocation ID

        Return
            (dict): The EIP record dict
        """
        if self._is_path(alloc_id):
            alloc_id = self._read_path(alloc_id)

        ip = aws.client('ec2').describe_addresses(AllocationIds=[alloc_id])

        if ip.get('Addresses'):
            return ip.get('Addresses')[0]

    def _is_path(self, path):
        """
        """
        if os.path.exists(path):
            return True
        return False

    def _read_path(self, path):
        """
        """
        with open(path, 'r') as f:
            return f.read().strip()



import pytest
import boto3
from pytest_mock import mocker
from ec2ools import aws

class Test:


    def test_get_region(self, mocker):
        """
        """
        co_mock = mocker.patch('ec2ools.aws.subprocess.check_output')
        co_mock.return_value = '{"region": "us-compton-1"}'

        assert aws.REGION is None
        assert aws.get_region() == 'us-compton-1'
        assert aws.REGION == 'us-compton-1'
        assert co_mock.call_count == 1

        # test caching
        assert aws.get_region() == 'us-compton-1'
        assert co_mock.call_count == 1


    def test_client(self, mocker):
        """
        """
        pass

    def test_metadata(self, mocker):
        """
        """
        co_mock = mocker.patch('ec2ools.aws.subprocess.check_output')
        co_mock.return_value = b'instance-id: i-testid'

        assert aws.metadata('instance-id') == 'i-testid'



    def test_metadata_exception(self, mocker):
        """
        """
        co_mock = mocker.patch('ec2ools.aws.subprocess.check_output')
        co_mock.return_value = '{"region": "us-compton-1"}'



from ec2ools.aws import MetaService, client

class Instance:

    def __init__(self, instance_id):
        super().__init__()
        self.instance_id = instance_id
        self.asg_name = None
        self.tags = []

    @classmethod
    def from_metadata(cls):
        """ Alternate constructor to instantiate
            an Instance instance from ec2 metadata
            service
        """
        ms = MetaService()
        inst_id = ms.query_meta('instance-id')

        new = cls(inst_id)

        return new

    def load_tags(self):
        """ load ec2 instance tags.
            check for and set the following tags as props
            - autoscaling group name
        """
        if len(self.tags) > 0:
            return

        ec2c = client("ec2")

        res = ec2c.describe_tags(Filters=[
            {
                'Name': 'resource-id',
                'Values': [self.instance_id]
            }
        ])

        for t in res['Tags']:
            # we did filter but lets dbl check
            if t['ResourceId'] != self.instance_id:
                continue
            self.tags.append(t)
            if t['Key'] == 'aws:autoscaling:groupName':
                self.asg_name = t['Value']

    def return_my_asg(self):
        """ find and return an asg
            instance this instance
            is a member of
        """
        self.load_tags()



# -*- coding: utf-8 -*-

"""Console script for ec2ools."""
import sys
import click
from ec2ools.aws.eip import EipBase


@click.group()
def main(args=None):
    pass


@main.command(name='allocate')
@click.argument('id',
              type=str)
def eip_allocate(**kw):

    print(kw)

    eip = EipBase()
    eip.allocate_eip(kw.get('id'))

if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



# -*- coding: utf-8 -*-

"""Console script for ec2ools."""
import sys
import json
import click
from ec2ools.aws.asg import (
    Asg
)
from ec2ools.aws import (
    ec2
)
ctx_settings = {
    'help_option_names': ['-h', '--help']
}

@click.group(context_settings=ctx_settings)
def main():
    pass


@main.command(name='is-oldest-instance', help=("Check if the instance is "
                                               "the oldest and exit 0 if "
                                               "true and exit 1 if not. "
                                               "If --instance_id/-i is not "
                                               "specified the current instanceId "
                                               "will be used."))
@click.option('--instance_id', '-i', type=str)
def is_oldest_instance(**kw):
    """
    """
    asg = Asg()

    res = asg.is_oldest_instance(kw.get('instance_id'))

    if not res:
        exit(1)
    exit(0)

@main.command('get-instances')
def get_asg_instances(**kw):
    """
    """
    id = 'i-0523e07206641cf70'
    # inst = ec2.Instance.from_metadata()
    inst = ec2.Instance(id)
    inst.load_tags()

    asg = Asg(inst.asg_name)
    asg_find = asg.get_instances()

    from pprint import pprint
    pprint(asg_find)


@main.command(name='get-asg')
def get_asg(**kw):
    pass




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt', 'r') as reqs:
    requirements = reqs.read().split('\n')

test_requirements = ['pytest', ]

setup(
    author="John Hardy",
    author_email='john@johnchardy.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Utilities to manage ec2 instances and asg's",
    entry_points={
        'console_scripts': [
            'ec2ools=ec2ools.cli:main',
            'ec2ools-eip=ec2ools.cli_eip:main',
            'ec2ools-asg=ec2ools.cli_asg:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='ec2ools',
    name='ec2ools',
    packages=find_packages(),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/ibejohn818/ec2ools',
    version='0.2.5',
    zip_safe=False,
)



import os
import requests
import boto3
import subprocess
import shlex
import json

EC2_METADATA='/usr/bin/ec2-metadata'
REGION=None

def client(name, region=None):
    """
    """
    if not region:
        region = get_region()
    return boto3.client(name, region_name=region)


def get_region():
    """ Get region from metadata
        and cache value in global
        constant
    """
    global REGION

    if os.environ.get("AWS_REGION"):
        REGION = os.environ.get("AWS_REGION")

    if REGION is None:
        cmd = "curl -s http://169.254.169.254/latest/dynamic/instance-identity/document"
        res = subprocess.check_output(shlex.split(cmd))
        REGION= json.loads(res)['region']
    return REGION


def metadata(name):
    """
    """
    arg = "--{}".format(name)

    cmd = "{} {}".format(EC2_METADATA, arg)

    try:
        res = subprocess.check_output(shlex.split(cmd))
        return res.decode().split(":")[1].strip()
    except Exception as e:
        print(str(e))



METADATA_HOST = "169.254.169.254"


class MetaService:

    _SESSION = None

    def __init__(self, revision='latest'):
        self.revision = revision

    @property
    def session(self):
        if MetaService._SESSION is None:
            sess = requests.Session()
            MetaService._SESSION = sess
        return MetaService._SESSION

    @property
    def url(self):
        return "http://{}/{}".format(
            METADATA_HOST,
            self.revision)

    def query(self, path):
        """ query path from the meta data service
        """
        req = self.session.get("{}/{}".format(self.url, path))

        if req.status_code != 200:
            raise Exception("Metadata query error: ", path)

        return req.text

    def query_meta(self, path):
        """ query a path from the meta-data namespace
        """
        res = self.query("meta-data/{}".format(path))
        return res

    def identity_doc(self):
        """ query the instance identity document
            and return json as a dict
        """
        res = self.query("dynamic/instance-identity/document")
        return json.loads(res)
